﻿using System;
using System.Collections.Generic;
using System.IO;
using System.IO.Ports;
using System.Linq;
using System.Timers;

namespace MellotProgrammer
{
    public class Connector
    {
        private SerialPort _serialPort;
        private readonly IDataOutput _dataOutput;
        private DumpHandler _dumpHandler;

        public Connector(IDataOutput dataOutput)
        {
            _dataOutput = dataOutput;
            _dumpHandler = new DumpHandler();
        }

        public void Connect(string comPort)
        {
            _serialPort = new SerialPort(comPort, 500000, Parity.None, 8, StopBits.One);
            _serialPort.DataReceived += Port_DataReceived;
            _serialPort.Open();
        }

        public IList<string> GetComPortNames()
        {
            return SerialPort.GetPortNames().Distinct().ToList();
        }

        public void PrintMenu()
        {
            WriteToPort("<PM>");
        }

        public void Dump()
        {
            WriteToPort("<RDF>");
        }

        public void DumpRam()
        {
            WriteToPort("<RDR>");
        }

        public bool IsConnected()
        {
            return _serialPort != null && _serialPort.IsOpen;
        }

        private void Port_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            var readData = _serialPort.ReadExisting();
            _dataOutput.OutputData(readData);
            HandleDataDumpIfNeeded(readData);
        }

        private void HandleDataDumpIfNeeded(string data)
        {
            _dumpHandler.HandleNewData(data);
        }

        public void Flash()
        {
            WriteToPort("<AUTO>");
        }

        public void SendData(string data)
        {
            WriteToPort(data);
        }

        private void WriteToPort(string data)
        {
            if (IsConnected())
            {
                _serialPort.Write(data);
            }
        }

        public void PrintVersion()
        {
            WriteToPort("<VER>");
        }

        public void EraseFlash()
        {
            WriteToPort("<ERF>");
        }

        public void EraseRam()
        {
            WriteToPort("<ERR>");
        }

        public void Restart()
        {
            WriteToPort("<HRST>");
        }

        public void DumpRomToFile(string saveFilePath)
        {
            _dumpHandler.StartDumpHandling(saveFilePath);
            WriteToPort("<PRTF>");
        }

        public void DumpRamToFile(string saveFilePath)
        {
            _dumpHandler.StartDumpHandling(saveFilePath);
            WriteToPort("<PRTR>");
        }
    }
}
