﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Timers;

namespace MellotProgrammer
{
    class DumpHandler
    {
        private Timer _timer;
        private readonly IList<string> _dataStorage;
        private int _dataStorageCounter;
        private string _saveFilePath;

        public DumpHandler()
        {
            _dataStorage = new List<string>();
            SetupDataStorageTimer();
        }

        private void SetupDataStorageTimer()
        {
            _dataStorageCounter = 0;
            _timer = new Timer(1000);
            _timer.Elapsed += Timer_Elapsed;
        }

        public void StartDumpHandling(string saveFilePath)
        {
            _saveFilePath = saveFilePath;
            _dataStorage.Clear();
            _timer.Start();
        }

        private void Timer_Elapsed(object sender, ElapsedEventArgs e)
        {
            if (_dataStorageCounter != _dataStorage.Count)
            {
                _dataStorageCounter = _dataStorage.Count;
            }
            else
            {
                _timer.Stop();
                WriteBinary();
            }
        }

        private void WriteBinary()
        {
            var data = string.Join("", _dataStorage);
            var lines = data.Split(new[] { "\r\n" }, StringSplitOptions.None)
                .Select(x => x.Replace(".", ""))
                .Where(x => x.Length == 57)
                .Select(x => x.Substring(9))
                .ToList();

            var hexString = string.Join("", lines).Replace(" ", "");
            var binaryData = StringToByteArrayFastest(hexString);
            File.WriteAllBytes(_saveFilePath, binaryData);
        }

        public static byte[] StringToByteArrayFastest(string hex)
        {
            byte[] binaryData = new byte[hex.Length >> 1];

            for (int i = 0; i < hex.Length / 2; i++)
            {
                binaryData[i] = (byte)((GetHexVal(hex[i * 2]) << 4) + (GetHexVal(hex[(i * 2) + 1])));
            }

            return binaryData;
        }

        public static int GetHexVal(char hex)
        {
            int value = hex;
            return value - (value < 58 ? 48 : (value < 97 ? 55 : 87));
        }

        public void HandleNewData(string data)
        {
            if (_timer.Enabled)
            {
                _dataStorage.Add(data);
            }
        }
    }
}
